#include "gloobus-preview-interface-document.h"


// @param obj This is the pointer to the class that will call the function load2
//
// Trick to launch a member function from pthread
void *LaunchMemberFunction(void *obj)
{
    iDocument *myObj = reinterpret_cast<iDocument *>(obj);
    myObj->load2();
    return 0;
};

// ========================== DESTRUCTOR ========================= //
iDocument::~iDocument()
{
	//Destroy plugin
}
//=========================  END ========================================= //
void iDocument::end()
{
	g_debug("Document END\n");
	
	m_destroyed = true;

	for(unsigned int i = 0; i< m_widget_pages.size(); i++)
	{
		g_print("destroing page %i of %i \n", i , m_widget_pages.size());
		gtk_widget_destroy(m_widget_pages[i]);
	}

	gtk_widget_destroy(m_vBox);
	gtk_widget_destroy(m_scwin);
}
// ========================= LOAD ================================= //
void iDocument::load(void)
{
	printf("Loading Document (%i Pages)\n", get_n_pages ());
	printf("Page size: %ix%i\n",get_width(),get_height());
		
	m_scwin = gtk_scrolled_window_new				( NULL, NULL );
	m_hadj  = gtk_scrolled_window_get_hadjustment 	( GTK_SCROLLED_WINDOW(m_scwin));
	m_vBox  = gtk_vbox_new 							( false, 1);
	
	m_destroyed = false;
	pthread_create(&m_thread, NULL, LaunchMemberFunction, this);
	
};
// ========================= LOAD 2 ( CALLED AS A THREAD) ================================= //
void iDocument::load2(void)
{
	GdkPixbuf *temp;
	
	//g_debug("THREAD START!!\n");
	for ( int i = 0; i < get_n_pages( ); i++)
	{	
		
		if(m_destroyed) 
		{ 
			g_debug("Document Destroyed: QUITING THREAD\n"); 
			pthread_exit(NULL); 
		}

		//g_debug("Loading page %i \n", i);
		temp = get_page_pixbuf 		( i ) ;
		m_widget_pages.push_back	( gtk_image_new_from_pixbuf (temp) );
	
	
    	gdk_threads_enter();		
			gtk_container_add			( GTK_CONTAINER(m_vBox), m_widget_pages[i] );
			gtk_adjustment_set_value	( m_hadj, gtk_adjustment_get_upper (m_hadj)/4);	//Scroll allways at the center
			gtk_widget_show_all			( m_scwin  );		//Upload the view
    	gdk_threads_leave();

		//g_debug("Page Loaded %i \n", i);
		
	}
	g_debug("Document Loaded!\n");
	pthread_exit(NULL);

}
// =========================== GET SIZE (W)================================= //
int  iDocument::get_width()
{
	return get_page_width()+24; //24 = Scrollbar
}

// =========================== GET SIZE (H)================================= //
int  iDocument::get_height()
{
	return get_page_height();
};

// =========================== DRAW DOCUMENT ================================= //
void iDocument::draw_doc(GtkContainer * container)
{
	
	gtk_scrolled_window_set_policy 		( GTK_SCROLLED_WINDOW(m_scwin ) ,GTK_POLICY_AUTOMATIC ,GTK_POLICY_AUTOMATIC );

	while(m_widget_pages.size () < 1 ) { ; }
	
	gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW(m_scwin ), m_vBox );
	gtk_container_add			   		  ( GTK_CONTAINER(container),	m_scwin );

    gtk_widget_set_uposition	  	( m_scwin , SHADOW_WIDTH, HEADER_HEIGHT + SHADOW_WIDTH );
    gtk_widget_set_usize			( m_scwin , get_width(), get_height());
    gtk_widget_show_all				( m_scwin  );
};

// ========================= DRAW DOC INFO ===================== //
void iDocument::draw_file_info(GtkContainer * container)
{
	int x,y;
	char buff[10];
	x = get_width();
	y = get_height();

	string text = get_file_info_string();
	sprintf(buff,"%2.f",(double) get_n_pages() );
	text = text + "Pages: " + buff;	
	
	draw_properties ( container, text, 25 + SHADOW_WIDTH, y + SHADOW_WIDTH+HEADER_HEIGHT);

}
// ========================= GET SCALE ===================== //
float iDocument::get_scale   ( int x, int y )
{

	float scale = 1;
	if ( x > y )
	{
		scale = get_screen_width()-200 / x;
	}	
	else
	{
		scale = get_screen_height()-200 / y;
	}
	return scale;
}

// ========================= DRAW ===================== //
// Virtual function that draws what plugin wants
void iDocument::draw( GtkContainer * container )
{
	draw_doc		( GTK_CONTAINER(container ));
	draw_file_info	( GTK_CONTAINER(container ));
}
