#include "gloobus-preview-interface-image.h"

// ========================= DESTRUCTOR ================================== //
iImage::~iImage()
{
	gtk_widget_destroy	(m_image);
	gtk_widget_destroy	(m_image_container);
	gdk_pixbuf_unref 	(m_pixbuf);
}
//=========================  END ========================================= //
void iImage::end()
{

}
// ========================= CONSTRUCTOR ================================= //
void iImage::load()
{
	g_debug("\nLoading Image...\n");
	scale_image(get_screen_width()-150,get_screen_height()-150);
};

// =========================== GET SIZE (W)================================= //
int  iImage::get_width()
{
	//g_debug("Width: %d\n", gdk_pixbuf_get_width(m_pixbuf));
	if(gdk_pixbuf_get_width(m_pixbuf) < 300)
	{
	 	return 300;
	}
	return gdk_pixbuf_get_width(m_pixbuf);
};

// =========================== GET SIZE (H)================================= //
int  iImage::get_height()
{
	//g_debug("Height: %d\n",gdk_pixbuf_get_height(m_pixbuf));
	if(gdk_pixbuf_get_height(m_pixbuf) < 300)
	{
		return 300;
	}
	return gdk_pixbuf_get_height(m_pixbuf);		//FIXME: Why do I have to add this +20?
};

// =========================== DRAW IMAGE ================================= //
void iImage::draw_image(GtkContainer * container)
{

	m_image 			= gtk_image_new_from_pixbuf (m_pixbuf);
	m_image_container	= gtk_fixed_new ();
	int x = SHADOW_WIDTH;			//Default values are at top left
	int y = HEADER_HEIGHT + SHADOW_WIDTH;			//and 20 from top (For the header)

	if(gdk_pixbuf_get_width(m_pixbuf) < 300)		//Minum width
	{
		x = (150 + SHADOW_WIDTH ) - gdk_pixbuf_get_width(m_pixbuf)/2;
	}
	if(gdk_pixbuf_get_height(m_pixbuf) < 300)		//Minium height
	{
		y = (150 + SHADOW_WIDTH + HEADER_HEIGHT) - gdk_pixbuf_get_height(m_pixbuf)/2;
	}


	gtk_fixed_put 	  (GTK_FIXED(m_image_container),m_image,x,y);
	gtk_container_add (GTK_CONTAINER(container),m_image_container);
	gtk_widget_show   (m_image_container);
	gtk_widget_show   (m_image);

};

// ========================== SCALE IMAGE ================================= //
// Scale the image to the maxium size of max_x and max_y
// TODO: Configure these values by a config file
void iImage::scale_image(int max_x, int max_y)
{
	g_debug("Scaling image...\n");
	GdkPixbuf 	*temp = get_pixbuf();				//Function Implemented by Plugin
	g_debug("Image Loaded\n");
	int x = gdk_pixbuf_get_width  (temp);
	int y = gdk_pixbuf_get_height (temp);

	bool modify = false;
	double value;
	
	if(x > max_x)
	{
		value = max_x/float(x);
		x = x*value;
		y = y*value;
		modify=true;
	}	
	if(y > max_y)  
	{
		value = max_y/float(y);
		x = x*value;
		y = y*value;
		modify=true;
	}

	if(modify)
	{
		value = double(x)/(gdk_pixbuf_get_width(temp));

		g_debug("Scaling Image (%f):%dx%d",value,gdk_pixbuf_get_width(temp),gdk_pixbuf_get_height(temp));
		g_debug("--> %ix%i\n",x,y);
	}
	m_pixbuf  = gdk_pixbuf_scale_simple (temp,x,y,GDK_INTERP_BILINEAR);
	gdk_pixbuf_unref (temp);
}
// ========================= DRAW IMAGE INFO ===================== //
void iImage::draw_file_info(GtkContainer * container)
{
	draw_properties ( container, get_file_info_string(), 25 + SHADOW_WIDTH, get_height() + SHADOW_WIDTH + HEADER_HEIGHT+5); //+5 = margin from the end of photo
}

// ========================= DRAW ===================== //
// Virtual function that draws what plugin wants
void iImage::draw( GtkContainer * container )
{
	draw_image		( GTK_CONTAINER(container ));
	draw_file_info	( GTK_CONTAINER(container ));
}
